@@ -44,7 +44,7 @@ int pth_usleep(unsigned int usec)
     static pth_key_t ev_key = PTH_KEY_INIT;
 
     /* calculate asleep time */
-    offset = pth_time(0, (long)usec);
+    offset = pth_time((long)(usec / 1000000), (long)(usec % 1000000));
     pth_time_set(&until, PTH_TIME_NOW);
     pth_time_add(&until, &offset);
 
