@@ -182,7 +182,8 @@ intern void *pth_scheduler(void *dummy)
          * Move threads from new queue to ready queue and give
          * them maximum priority so they start immediately
          */
-        while ((t = pth_pqueue_delmax(&pth_NQ)) != NULL) {
+        while ((t = pth_pqueue_tail(&pth_NQ)) != NULL) {
+            pth_pqueue_delete(&pth_NQ, t);
             t->state = PTH_STATE_READY;
             pth_pqueue_insert(&pth_RQ, pth_pqueue_favorite_prio(&pth_RQ), t);
             pth_debug2("pth_scheduler: new thread \"%s\" moved to top of ready queue", t->name);
