@@ -231,23 +231,29 @@ int pth_cond_init(pth_cond_t *cond)
 {
     if (cond == NULL)
         return_errno(FALSE, EINVAL);
-    cond->cn_state = PTH_COND_INITIALIZED;
+    cond->cn_state   = PTH_COND_INITIALIZED;
+    cond->cn_waiters = 0;
     return TRUE;
 }
 
-static void pth_cond_cleanup_handler(void *_mutex)
+static void pth_cond_cleanup_handler(void *_cleanvec)
 {
-    pth_mutex_t *mutex = (pth_mutex_t *)_mutex;
+    pth_mutex_t *mutex = (pth_mutex_t *)(((void **)_cleanvec)[0]);
+    pth_cond_t  *cond  = (pth_cond_t  *)(((void **)_cleanvec)[1]);
 
     /* re-acquire mutex when pth_cond_await() is cancelled
        in order to restore the condition variable semantics */
     pth_mutex_acquire(mutex, FALSE, NULL);
+
+    /* fix number of waiters */
+    cond->cn_waiters--;
     return;
 }
 
 int pth_cond_await(pth_cond_t *cond, pth_mutex_t *mutex, pth_event_t ev_extra)
 {
     static pth_key_t ev_key = PTH_KEY_INIT;
+    void *cleanvec[2];
     pth_event_t ev;
 
     /* consistency checks */
@@ -265,6 +271,9 @@ int pth_cond_await(pth_cond_t *cond, pth_mutex_t *mutex, pth_event_t ev_extra)
         return TRUE;
     }
 
+    /* add us to the number of waiters */
+    cond->cn_waiters++;
+
     /* release mutex (caller had to acquire it first) */
     pth_mutex_release(mutex);
 
@@ -272,7 +281,9 @@ int pth_cond_await(pth_cond_t *cond, pth_mutex_t *mutex, pth_event_t ev_extra)
     ev = pth_event(PTH_EVENT_COND|PTH_MODE_STATIC, &ev_key, cond);
     if (ev_extra != NULL)
         pth_event_concat(ev, ev_extra, NULL);
-    pth_cleanup_push(pth_cond_cleanup_handler, mutex);
+    cleanvec[0] = mutex;
+    cleanvec[1] = cond;
+    pth_cleanup_push(pth_cond_cleanup_handler, cleanvec);
     pth_wait(ev);
     pth_cleanup_pop(FALSE);
     if (ev_extra != NULL)
@@ -280,6 +291,11 @@ int pth_cond_await(pth_cond_t *cond, pth_mutex_t *mutex, pth_event_t ev_extra)
 
     /* reacquire mutex */
     pth_mutex_acquire(mutex, FALSE, NULL);
+
+    /* remove us from the number of waiters */
+    cond->cn_waiters--;
+
+    /* release mutex (caller had to acquire it first) */
     return TRUE;
 }
 
@@ -291,16 +307,19 @@ int pth_cond_notify(pth_cond_t *cond, int broadcast)
     if (!(cond->cn_state & PTH_COND_INITIALIZED))
         return_errno(FALSE, EDEADLK);
 
-    /* signal the condition */
-    cond->cn_state |= PTH_COND_SIGNALED;
-    if (broadcast)
-        cond->cn_state |= PTH_COND_BROADCAST;
-    else
-        cond->cn_state &= ~(PTH_COND_BROADCAST);
-    cond->cn_state &= ~(PTH_COND_HANDLED);
+    /* do something only if there is at least one waiters (POSIX semantics) */
+    if (cond->cn_waiters > 0) {
+        /* signal the condition */
+        cond->cn_state |= PTH_COND_SIGNALED;
+        if (broadcast)
+            cond->cn_state |= PTH_COND_BROADCAST;
+        else
+            cond->cn_state &= ~(PTH_COND_BROADCAST);
+        cond->cn_state &= ~(PTH_COND_HANDLED);
 
-    /* and give other threads a chance to awake */
-    pth_yield(NULL);
+        /* and give other threads a chance to awake */
+        pth_yield(NULL);
+    }
 
     /* return to caller */
     return TRUE;
