@@ -72,7 +72,7 @@ intern void pth_time_usleep(unsigned long usec)
 #endif /* cpp */
 
 /* time value constructor */
-pth_time_t pth_time(int sec, int usec)
+pth_time_t pth_time(long sec, long usec)
 {
     pth_time_t tv;
 
@@ -82,7 +82,7 @@ pth_time_t pth_time(int sec, int usec)
 }
 
 /* timeout value constructor */
-pth_time_t pth_timeout(int sec, int usec)
+pth_time_t pth_timeout(long sec, long usec)
 {
     pth_time_t tv;
     pth_time_t tvd;
