@@ -38,7 +38,7 @@ static void *my_reader(void *_arg)
     char buf[3];
     int n;
 
-    while (1) {
+    for (;;) {
         n = pth_read(STDIN_FILENO, buf, 1);
         if (n < 0) {
             fprintf(stderr, "reader: error\n");
