@@ -50,7 +50,7 @@ typedef enum {
     eating
 } philstat;
 
-static char *philstatstr[3] = {
+static const char *philstatstr[3] = {
     "thinking",
     "hungry  ",
     "EATING  "
@@ -76,7 +76,7 @@ static void printstate(void)
     return;
 }
 
-static int test(int i)
+static int test(unsigned int i)
 {
     if (   ((tab->status)[i] == hungry)
         && ((tab->status)[(i + 1) % PHILNUM] != eating)
@@ -88,7 +88,7 @@ static int test(int i)
     return FALSE;
 }
 
-static void pickup(int k)
+static void pickup(unsigned int k)
 {
     pth_mutex_acquire(&(tab->mutex), FALSE, NULL);
     (tab->status)[k] = hungry;
@@ -100,7 +100,7 @@ static void pickup(int k)
     return;
 }
 
-static void putdown(int k)
+static void putdown(unsigned int k)
 {
     pth_mutex_acquire(&(tab->mutex), FALSE, NULL);
     (tab->status)[k] = thinking;
@@ -113,7 +113,7 @@ static void putdown(int k)
 
 static void *philosopher(void *_who)
 {
-    int *who = (int *)_who;
+    unsigned int *who = (unsigned int *)_who;
 
     /* For simplicity, all philosophers eat for the same amount of time
        and think for a time that is simply related to their position at
